#include "Color.h"

#include <unordered_map>

// Ref https://m3.material.io/styles/color/the-color-system/tokens

enum class ColorPaletteToken
{
    Primary_10,
    Primary_20,
    Primary_30,
    Primary_40,
    Primary_80,
    Primary_90,
    Primary_100,
    Secondary_10,
    Secondary_20,
    Secondary_30,
    Secondary_40,
    Secondary_80,
    Secondary_90,
    Secondary_100,
    Tertiary_10,
    Tertiary_20,
    Tertiary_30,
    Tertiary_40,
    Tertiary_80,
    Tertiary_90,
    Tertiary_100,
    Neutral_0,
    Neutral_10,
    Neutral_20,
    Neutral_30,
    Neutral_80,
    Neutral_90,
    Neutral_95,
    Neutral_99,
    Neutral_Variant_30,
    Neutral_Variant_50,
    Neutral_Variant_60,
    Neutral_Variant_80,
    Neutral_Variant_90,
    Error_10,
    Error_20,
    Error_30,
    Error_40,
    Error_80,
    Error_90,
    Error_100
};

std::unordered_map<ColorPaletteToken, std::string> mPaletteColors = {
    {ColorPaletteToken::Primary_10, "#21005E"},
    {ColorPaletteToken::Primary_20, "#371E73"},
    {ColorPaletteToken::Primary_30, "#4F378B"},
    {ColorPaletteToken::Primary_40, "#6750A4"},
    {ColorPaletteToken::Primary_80, "#D0BCFF"},
    {ColorPaletteToken::Primary_90, "#EADDFF"},
    {ColorPaletteToken::Primary_100, "#FFFFFF"},
    {ColorPaletteToken::Secondary_10, "#1E192B"},
    {ColorPaletteToken::Secondary_20, "#332D41"},
    {ColorPaletteToken::Secondary_30, "#4A4458"},
    {ColorPaletteToken::Secondary_40, "#625B71"},
    {ColorPaletteToken::Secondary_80, "#CCC2DC"},
    {ColorPaletteToken::Secondary_90, "#E8DEF8"},
    {ColorPaletteToken::Secondary_100, "#FFFFFF"},
    {ColorPaletteToken::Tertiary_10, "#370B1E"},
    {ColorPaletteToken::Tertiary_20, "#492532"},
    {ColorPaletteToken::Tertiary_30, "#633B48"},
    {ColorPaletteToken::Tertiary_40, "#7D5260"},
    {ColorPaletteToken::Tertiary_80, "#EFB8C8"},
    {ColorPaletteToken::Tertiary_90, "#FFD8E4"},
    {ColorPaletteToken::Tertiary_100, "#FFFFFF"},
    {ColorPaletteToken::Neutral_0, "#000000"},
    {ColorPaletteToken::Neutral_10, "#1C1B1F"},
    {ColorPaletteToken::Neutral_20, "#313033"},
    {ColorPaletteToken::Neutral_90, "#E6E1E5"},
    {ColorPaletteToken::Neutral_95, "#F4EFF4"},
    {ColorPaletteToken::Neutral_99, "#FFFBFE"},
    {ColorPaletteToken::Neutral_Variant_30, "#49454F"},
    {ColorPaletteToken::Neutral_Variant_50, "#79747E"},
    {ColorPaletteToken::Neutral_Variant_60, "#938F99"},
    {ColorPaletteToken::Neutral_Variant_80, "#CAC4D0"},
    {ColorPaletteToken::Neutral_Variant_90, "#E7E0EC"},
    {ColorPaletteToken::Error_10, "#410E0B"},
    {ColorPaletteToken::Error_20, "#601410"},
    {ColorPaletteToken::Error_30, "#8C1D18"},
    {ColorPaletteToken::Error_40, "#B3261E"},
    {ColorPaletteToken::Error_80, "#F2B8B5"},
    {ColorPaletteToken::Error_90, "#F9DEDC"},
    {ColorPaletteToken::Error_100, "#FFFFFF"},
};

enum class SystemToken
{
    Primary,
    Primary_Container,
    Secondary,
    Secondary_Container,
    Tertiary,
    Tertiary_Container,
    Surface,
    Surface_Variant,
    Background,
    Error,
    Error_Container,
    On_Primary,
    On_Primary_Container,
    On_Secondary,
    On_Secondary_Container,
    On_Tertiary,
    On_Tertiary_Container,
    On_Surface,
    On_Surface_Variant,
    On_Error,
    On_Error_Container,
    On_Background,
    Outline,
    Ouline_Variant,
    Shadow,
    Surface_Tint_Color,
    Inverse_Surface,
    Inverse_On_Surface,
    Inverse_Primary,
    Scrim
};

std::unordered_map<SystemToken, ColorPaletteToken> mLightTheme = {
    {SystemToken::Primary, ColorPaletteToken::Primary_40},
    {SystemToken::Primary_Container, ColorPaletteToken::Primary_90},
    {SystemToken::Secondary, ColorPaletteToken::Secondary_40},
    {SystemToken::Secondary_Container, ColorPaletteToken::Secondary_90},
    {SystemToken::Tertiary, ColorPaletteToken::Tertiary_40},
    {SystemToken::Tertiary_Container, ColorPaletteToken::Tertiary_90},
    {SystemToken::Surface, ColorPaletteToken::Neutral_99},
    {SystemToken::Surface_Variant, ColorPaletteToken::Neutral_Variant_90},
    {SystemToken::Background, ColorPaletteToken::Neutral_99},
    {SystemToken::Error, ColorPaletteToken::Error_40},
    {SystemToken::Error_Container, ColorPaletteToken::Error_90},
    {SystemToken::On_Primary, ColorPaletteToken::Primary_100},
    {SystemToken::On_Primary_Container, ColorPaletteToken::Primary_10},
    {SystemToken::On_Secondary, ColorPaletteToken::Secondary_100},
    {SystemToken::On_Secondary_Container, ColorPaletteToken::Secondary_10},
    {SystemToken::On_Tertiary, ColorPaletteToken::Tertiary_100},
    {SystemToken::On_Tertiary_Container, ColorPaletteToken::Tertiary_10},
    {SystemToken::On_Surface, ColorPaletteToken::Neutral_10},
    {SystemToken::On_Surface_Variant, ColorPaletteToken::Neutral_Variant_30},
    {SystemToken::On_Error, ColorPaletteToken::Error_100},
    {SystemToken::On_Error_Container, ColorPaletteToken::Error_10},
    {SystemToken::On_Background, ColorPaletteToken::Neutral_10},
    {SystemToken::Outline, ColorPaletteToken::Neutral_Variant_50},
    {SystemToken::Ouline_Variant, ColorPaletteToken::Neutral_Variant_80},
    {SystemToken::Shadow, ColorPaletteToken::Neutral_0},
    {SystemToken::Surface_Tint_Color, ColorPaletteToken::Primary_40},
    {SystemToken::Inverse_Surface, ColorPaletteToken::Neutral_20},
    {SystemToken::Inverse_On_Surface, ColorPaletteToken::Neutral_95},
    {SystemToken::Inverse_Primary, ColorPaletteToken::Primary_80},
    {SystemToken::Scrim, ColorPaletteToken::Neutral_0},
};

class Theme
{
public:
    static Color getBackgroundPrimary();

    static Color getBannerBackground();

    static Color getButtonPrimaryBackground();

    static Color getButtonPrimaryPressed();

    static Color getTextPrimary();
};
